@@ -28,6 +28,8 @@ from typing import Dict
 from typing import Tuple
 from typing import List
 from typing import Iterable
+from typing import Any
+from typing import Type
 import os
 import sys
 import unicodedata
@@ -77,12 +79,30 @@ except (ImportError,):
 # letter of a word until the candidate lookup table pops up.
 MAX_WORDS = 100
 
-class Dictionary:
-    '''A class to hold a hunspell dictionary
-    '''
-    def __init__(self, name: str = 'en_US') -> None:
+class Dictionary():
+    '''A class to hold a hunspell dictionary'''
+    # As soon as it is OK ot require Python >= 3.7, use these nicer type hints:
+    #
+    # _instances: Dict[Tuple[Type['Dictionary'], str], 'Dictionary'] = {}
+    #
+    # def __new__(cls: Type['Dictionary'], name: str = 'en_US') -> 'Dictionary':
+    #
+    # openSUSE Leap 15.4 still has only Python 3.6.
+    _instances: Dict[Tuple[Any, str], Any] = {}
+
+    def __new__(cls: Any, name: str = 'en_US') -> Any:
+        '''Caching instances of this class and reuse previously created instances'''
+        key = (cls, name)
+        if key not in cls._instances:
+            instance = super().__new__(cls)
+            cls._instances[key] = instance
+            instance._initialize(name=name)
+        return cls._instances[key]
+
+    def _initialize(self, name: str = 'en_US') -> None:
+        '''Initialize a Dictionary'''
         if DEBUG_LEVEL > 1:
-            LOGGER.debug('Dictionary.__init__(name=%s)\n', name)
+            LOGGER.debug('name=%s', name)
         self.name = name
         self.language = self.name.split('_')[0]
         self.dic_path = ''
@@ -504,15 +524,16 @@ class Hunspell:
                 match_list.append(dictionary.name)
         return match_list
 
-    def spellcheck_single_dictionary(self, words: Iterable[str] = ()) -> bool:
+    def spellcheck_single_dictionary(self, words: Iterable[str] = ()) -> List[str]:
         '''
         Checks whether there is at least one dictionary where all words
         in the input list spellcheck as True.
 
         :param words: A list or tuple of words to spellcheck
-        :return: - False if at least one input word is empty
-                 - True if there is at least one dictionary where all
-                   words in the input list spellcheck as True, else False.
+        :return: - '' if at least one input word is empty
+                 - name of a dictionary where all
+                   words in the input list spellcheck as True, if
+                   there is no such dictionary, return ''.
 
         Examples:
 
@@ -521,49 +542,54 @@ class Hunspell:
         All 3 words spellcheck True in the French dictionary:
 
         >>> h.spellcheck_single_dictionary(('Je', 'suis', 'arrivé'))
-        True
+        ['fr_FR']
 
         Not a correct French sentence but all 3 words spellcheck True
         in the French dictionary:
 
         >>> h.spellcheck_single_dictionary(('Je', 'suis', 'arrive'))
-        True
+        ['fr_FR']
 
+        All words spellcheck as True in both the English and the French dictionary:
+
+        >>> h.spellcheck_single_dictionary(('arrive', 'arrive', 'arrive'))
+        ['en_US', 'fr_FR']
 
         Not all 3 words spellcheck True in the French dictionary and
         not all 3 words spellcheck True in the English dictionary either:
 
         >>> h.spellcheck_single_dictionary(('Je', 'suis', 'arrived'))
-        False
+        []
 
         All 3 words spellcheck True in the English dictionary:
 
         >>> h.spellcheck_single_dictionary(('I', 'have', 'arrived'))
-        True
+        ['en_US']
 
         Unfortunately these 3 words spellcheck True in the French dictionary
         ('have' is a form of the verb 'haver', 'I' is the name of the letter):
 
         >>> h.spellcheck_single_dictionary(('I', 'have', 'arrivé'))
-        True
+        ['fr_FR']
 
-        If the input contains empty words the result is False:
+        If the input contains empty words the result is '':
 
         >>> h.spellcheck_single_dictionary(('I', 'have', ''))
-        False
+        []
         '''
         if not words:
-            return False
+            return []
         for word in words:
             if not word:
-                return False
+                return []
+        dictionary_names = []
         for dictionary in self._dictionaries:
             spellcheck_total = True
             for word in words:
                 spellcheck_total &= dictionary.spellcheck(word)
             if spellcheck_total:
-                return True
-        return False
+                dictionary_names.append(dictionary.name)
+        return sorted(dictionary_names)
 
     def suggest(self, input_phrase: str) -> List[Tuple[str, int]]:
         # pylint: disable=line-too-long
@@ -753,6 +779,11 @@ def main() -> None:
     runs some tests and prints profiling data.
     '''
     log_handler = logging.StreamHandler(stream=sys.stderr)
+    log_formatter = logging.Formatter(
+        '%(asctime)s %(filename)s '
+        'line %(lineno)d %(funcName)s %(levelname)s: '
+        '%(message)s')
+    log_handler.setFormatter(log_formatter)
     LOGGER.setLevel(logging.DEBUG)
     LOGGER.addHandler(log_handler)
 
